@@ -14,8 +14,6 @@ strings, URLs etc.
 """
 
 
-from __future__ import with_statement
-
 import sys, os, os.path, warnings, cStringIO, codecs, pyexpat, contextlib
 
 from xml.parsers import expat
@@ -130,6 +128,9 @@ class SGMLOPParser(Parser):
 
 
 class ExpatParser(Parser):
+	"""
+	A parser using Pythons builtin :mod:`expat` XML parser.
+	"""
 	def __init__(self, encoding=None, transcode=False, xmldecl=False, doctype=False):
 		Parser.__init__(self)
 		self.encoding = encoding
@@ -246,7 +247,7 @@ class Builder(object):
 			the appropriate interface).
 
 		:var:`prefixes` : mapping
-			a mapping that maps namespace prefixes to namespace names/modules)
+			a mapping that maps namespace prefixes to namespace names/modules
 			(or lists of namespace names/modules). This is used to preinitialize
 			the namespace prefix mapping.
 
@@ -263,12 +264,12 @@ class Builder(object):
 			Should the parsed XML nodes be validated after parsing?
 
 		:var:`encoding` : string or :const:`None`
-			The default encoding to use, when the source doesn't provide an
+			The default encoding to use when the source doesn't provide an
 			encoding. The default :const:`None` results in the encoding being
 			detected from the XML itself.
 
 		:var:`pool` : :class:`ll.xist.xsc.Pool` object
-			This pool will be used for creates all nodes during parsing.
+			This pool will be used for creating all nodes during parsing.
 		"""
 		self.parser = parser
 
@@ -362,15 +363,18 @@ class Builder(object):
 
 		self.base = url.URL(base)
 
-		try:
-			olddefault = libxml2.lineNumbersDefault(1)
-			doc = libxml2.htmlReadMemory(data, len(data), sysid, encoding, 0x160)
+		if not data:
+			node = xsc.Frag()
+		else:
 			try:
-				node = toxsc(doc)
+				olddefault = libxml2.lineNumbersDefault(1)
+				doc = libxml2.htmlReadMemory(data, len(data), sysid, encoding, 0x160)
+				try:
+					node = toxsc(doc)
+				finally:
+					doc.freeDoc()
 			finally:
-				doc.freeDoc()
-		finally:
-			libxml2.lineNumbersDefault(olddefault)
+				libxml2.lineNumbersDefault(olddefault)
 		return node
 
 	def _begin(self, base=None, encoding=None):
@@ -414,10 +418,10 @@ class Builder(object):
 
 	def parseiter(self, iterable, base=None, encoding=None):
 		"""
-		Parse the input from the iterable :var:`iterable:var:` (which must
-		produce the input in chunks of bytes) into an XIST tree. :var:`base` is
-		the base URL for the parsing process, :var:`encoding` can be used to
-		force the parser to use the specified encoding.
+		Parse the input from the iterable :var:`var` (which must produce the input
+		in chunks of bytes) into an XIST tree. :var:`base` is the base URL for the
+		parsing process, :var:`encoding` can be used to force the parser to use
+		the specified encoding.
 		"""
 		self.url = url.URL(base if base is not None else "ITER")
 		if self.tidy:
@@ -433,7 +437,7 @@ class Builder(object):
 		Parse XML input from the stream :var:`stream`. :var:`base` is the base
 		URL for the parsing process, :var:`encoding` can be used to force the
 		parser to use the specified encoding. :var:`bufsize` is the buffer size
-		used from reading the stream in blocks.
+		used for reading the stream in blocks.
 		"""
 		self.url = url.URL(base if base is not None else "STREAM")
 		parser = self._begin(base=base, encoding=encoding)
@@ -451,8 +455,8 @@ class Builder(object):
 		Parse XML input from the file named :var:`filename`. :var:`base` is the
 		base URL for the parsing process (defaulting to :var:`filename` if not
 		specified), :var:`encoding` can be used to force the parser to use the
-		specified encoding. :var:`bufsize` is the buffer size used from reading
-		the stream in blocks.
+		specified encoding. :var:`bufsize` is the buffer size used for reading
+		the file in blocks.
 		"""
 		self.url = url.File(filename)
 		if base is None:
@@ -476,7 +480,7 @@ class Builder(object):
 		base URL for the parsing process (defaulting to the final URL of the
 		response (i.e. including redirects)). :var:`encoding` can be used to
 		force the parser to use the specified encoding. :var:`bufsize` is the
-		buffer size used from reading the response in blocks :var:`args` and
+		buffer size used for reading the response in blocks. :var:`args` and
 		:var:`kwargs` will be passed on to the :meth:`open` call.
 		"""
 		name = url.URL(name)
@@ -712,9 +716,9 @@ def parseiter(iterable, base=None, encoding=None, **builderargs):
 def parsestream(stream, base=None, encoding=None, bufsize=8192, **builderargs):
 	"""
 	Parse XML from the stream :var:`stream` into an XIST tree. For the arguments
-	:var:`base`, :var:`encoding` and :var:`bufzise` see the method :meth:`parse`
-	in the :class:`Parser` class. You can pass any other argument that the
-	:class:`Builder` constructor takes as keyword arguments via
+	:var:`base`, :var:`encoding` and :var:`bufzise` see the method
+	:meth:`parsestream` in the :class:`Parser` class. You can pass any other
+	argument that the :class:`Builder` constructor takes as keyword arguments via
 	:var:`builderargs`.
 	"""
 	builder = Builder(**builderargs)
